;Sample POOL code, you may use it, modify it
;and distribute it under the GPL license.
;Authors: R&D Independent Team
;Source: www.pool-rnd.com

window ht
:pool_cfg,"refresh := false
:pool_cfg,"scale_x := 50
:pool_cfg,"scale_y := 50

;initial values of A, B, C coefficients:
"a := 1
"b := 1
"c := 0

to f :x ;f(x) = A*x^2 + B*x + C
  op :a * :x^2 + :b * :x + :c
end

;function that solves A*x^2 + B*x + C = 0
;output (array):
;{delta} - if A = 0 and B = 0 or discriminant < 0
;{delta, x1} - if A = 0 and B <> 0 (linear equation)
;{delta, x1, x2} - if discriminant >= 0
to solve
  let "d :b^2 - 4 * :a * :c ;delta (discriminant)
  if :a = 0 [
    ifelse :b = 0 [op (array :d)]
    [op array :d (-:c/:b)]
  ]
  if :d >= 0 [
    let "x1 (-:b - sqrt :d) / (2 * :a)
    let "x2 (-:b + sqrt :d) / (2 * :a)
    op (array :d :x1 :x2)
  ]
  op (array :d)
end

to circle :x ;draws a circle around the root (:x, 0)
  (setpc "red 50)
  pu setxy :x 0
  arc 360 10 / :pool_cfg,"scale_x ;radius adjusted to the drawing scale
end

to draw_f :x0 :x1 :n ;quadratic function plot
  setpc {30 0 20}
  pu setxy :x0 f :x0 pd
  for [x :x0 :x1 [(:x1 - :x0)/:n]] [setxy :x f :x]
end

to update
  clean
  draw_f :pool_inf,"wall_left :pool_inf,"wall_right 300
  let "r solve
  settext :ld word "|d = | :r,1
  ifelse count :r > 1 [
    settext :l1 word "|x1 = | :r,2
    circle :r,2
    ifelse count :r > 2 [
      circle :r,3
      settext :l2 word "|x2 = | :r,3
    ] [settext :l2 "|---|]
  ] [
    settext :l1 "|---|
    settext :l2 "|---|
  ]
  refresh
end

to onworldchanged
  update
end

;labels with the equation's roots and discriminant:
"l1 := label "x1 {70 6 200 24}
"l2 := label "x2 {70 34 200 24}
"ld := label "delta {70 62 200 24}

;sliders to change A, B i C coefficients,
;here you can change their ranges
"sa := (slider "A {70 100} {-10 10} :a)
setonchange :sa ["a := getvalue :sa update]
"sb := (slider "B {70 150} {-10 10} :b)
setonchange :sb ["b := getvalue :sb update]
"sc := (slider "C {70 200} {-5 5} :c)
setonchange :sc ["c := getvalue :sc update]

update

